<?php

/**
 * Backlinks plugin CLI commands
 *
 * ## EXAMPLES
 *
 *     wp backlinks update
 *
 * @when after_wp_load
 */

class BacklinksCLI {

    private function display_posts( $header, $posts ) {
        global $backlinks_plugin;

		$td = $backlinks_plugin->textdomain();

        $output = [];
		foreach ( $posts as $post ) {
			$output[] = [
				__('ID', $td) => $post->ID,
				__('Type', $td) => $post->post_type,
				__('Status', $td) => $post->post_status,
				__('Title', $td) => $post->post_title,
				__('Date', $td) => $post->post_date,
				// __('Link', $td) => get_permalink( $post ),
			];
		}

        if ( $output ) {
            WP_CLI::line( $header );
            WP_CLI\Utils\format_items( 'table', $output, join(',', array_keys($output[0])) );
        } else {
            WP_CLI::warning( sprintf( __( '%s - NONE', $td ), $header ) );
        }
    }

    /**
     * Update links for all posts
     *
     * ## OPTIONS
     *
     * [<post_id>...]
     * : Update only the given posts
     *
     * [--format=<format>]
     * : Render output in a particular format.
     * ---
     * default: table
     * options:
     *   - table
     *   - csv
     *   - json
     *   - count
     *   - yaml
     * ---
     *
     *
     * ## EXAMPLES
     *
     *     wp backlinks rescan
     *
     */

    public function rescan( $args, $assoc ) {
        global $backlinks_plugin;

		$td = $backlinks_plugin->textdomain();

        if ( empty( $args ) ) {
            $posts = get_posts( $backlinks_plugin->build_post_query() );
        } else {
            $posts = get_posts( $backlinks_plugin->build_post_query( [
                'post__in' => array_map( 'intval', array_filter( $args, 'intval' ) ),
                'post_status' => array_merge( [ 'draft' ], $backlinks_plugin->post_statuses() ),
            ] ) );
        }

		$output = [];

		foreach ( $posts as $post ) {
			$links = $backlinks_plugin->register_outlinks( $post );

			$output[] = [
				__('ID', $td) => $post->ID,
				__('Type', $td) => $post->post_type,
				__('Status', $td) => $post->post_status,
				__('Title', $td) => $post->post_title,
				__('Outgoing links', $td) => count( $links ),
			];
		}

		if ( $output ) {
			WP_CLI\Utils\format_items( $assoc['format'], $output, join(',', array_keys($output[0])) );
		}
    }



    /**
     * Show backlinks for all posts
     *
     * ## OPTIONS
     *
     * [--max=<limit>]
     * : Show only posts with less than <limit> backlinks
     *
     * [--format=<format>]
     * : Render output in a particular format.
     * ---
     * default: table
     * options:
     *   - table
     *   - csv
     *   - json
     *   - count
     *   - yaml
     * ---
     *
     *
     * ## EXAMPLES
     *
     *     wp backlinks status
     *
     */

    public function status( $args, $assoc ) {
        global $backlinks_plugin;

		$td = $backlinks_plugin->textdomain();

        $max = 9999;
        if ( isset( $assoc['max'] ) )
            $max = intval( $assoc['max'] );

        $posts = get_posts( $backlinks_plugin->build_post_query() );

		$pools = [];

		foreach ( $posts as $post ) {
			$outgoing = get_the_terms( $post, $backlinks_plugin->taxonomy() );
			if ( is_wp_error( $outgoing ) ) {
				WP_CLI::error( $outgoing );
				$outgoing = [];
			} elseif ( $outgoing === FALSE ) {
				$outgoing = [];
			}

            $incoming = $backlinks_plugin->get_backlinks( $post );

            $count = count( $incoming );
			$pools[ $count ][] = [
				__('ID', $td) => $post->ID,
				__('Type', $td) => $post->post_type,
				__('Status', $td) => $post->post_status,
				__('Title', $td) => $post->post_title,
				__('Incoming', $td) => $count,
				__('Outgoing', $td) => count( $outgoing ),
			];
        }

        ksort( $pools, SORT_NUMERIC );

        foreach ( $pools as $count => $output ) {
            if ( $count > $max )
                break;

            WP_CLI::line( sprintf( "Found %d posts with %d backlinks", count( $output ), $count ) );
            WP_CLI\Utils\format_items( $assoc['format'], $output, join(',', array_keys($output[0])) );
        }
    }


    /**
     * Show registed links for post
     *
     * ## OPTIONS
     *
     * <post_id>
     * : Show incoming and outgoing links for post_id
     *
     * [--format=<format>]
     * : Render output in a particular format.
     * ---
     * default: table
     * options:
     *   - table
     *   - csv
     *   - json
     *   - count
     *   - yaml
     * ---
     *
     * ## EXAMPLES
     *
     *     wp backlink show 42
     *
     * @alias thumb
     */

    public function show( $args, $assoc ) {
        global $backlinks_plugin;

		$td = $backlinks_plugin->textdomain();

        $posts = get_posts( $backlinks_plugin->build_post_query( [
            'post__in' => [ intval($args[0] ) ],
        ] ) );

		if ( !$posts ) {
			WP_CLI::error( 'No such eligible post' );
			return;
		}
        $post = $posts[0];

        $this->display_posts( __( 'This post', $td ) , [ $post ] );

		$outgoing = $backlinks_plugin->get_outlinks( $post );
        $header = sprintf( __('Found %d outgoing links.', $td), count( $outgoing ));
        $this->display_posts( $header , $outgoing );

		$incoming = $backlinks_plugin->get_backlinks( $post );
        $header = sprintf( __('Found %d incoming links.', $td), count( $incoming ));
        $this->display_posts( $header , $incoming );
    }



    /**
     * Suggest links for post
     *
     * ## OPTIONS
     *
     * <post-id>
     * : Suggest incoming and outgoing links for post-id
     *
     * ## EXAMPLES
     *
     *     wp backlink suggest 42
     *
     */

    public function suggest( $args, $assoc ) {
        global $backlinks_plugin;

		$td = $backlinks_plugin->textdomain();

        $posts = get_posts( $backlinks_plugin->build_post_query( [
            'post__in' => [ intval($args[0] ) ],
        ] ) );

		if ( !$posts ) {
			WP_CLI::error( 'No such eligible post' );
			return;
		}
        $post = $posts[0];

        $this->display_posts( __( 'This post', $td ) , [ $post ] );

        $outlinks = $backlinks_plugin->get_outlinks( $post );
        $this->display_posts( __( 'Outgoing links', $td ), $outlinks );

        $tag_suggest = $backlinks_plugin->backlink_suggestions_by_taxonomy( $post, 'post_tag', $outlinks );
        $this->display_posts( __( "Outgoing link suggestions based on tags", $td ), $tag_suggest );

        $cat_suggest = $backlinks_plugin->backlink_suggestions_by_taxonomy( $post, 'category', array_merge( $outlinks, $tag_suggest ) );
        $this->display_posts( __( "Outgoing link suggestions based on category", $td ), $cat_suggest );

        $backlinks = $backlinks_plugin->get_backlinks( $post );
        $this->display_posts( __( "Incoming links", $td ), $backlinks );

        $tag_suggest = $backlinks_plugin->backlink_suggestions_by_taxonomy( $post, 'post_tag', $backlinks );
        $this->display_posts( __( "Incoming link suggestions based on tags", $td ), $tag_suggest );

        $cat_suggest = $backlinks_plugin->backlink_suggestions_by_taxonomy( $post, 'category', array_merge( $backlinks, $tag_suggest ) );
        $this->display_posts( __( "Incoming link suggestions based on category", $td ), $cat_suggest );
    }
}

WP_CLI::add_command( 'backlinks', new BacklinksCLI() );
